@@ -28,7 +28,7 @@ VALID_BORDER: Final = {
     "wide",
 }
 VALID_EDGE: Final = {"top", "right", "bottom", "left", "none"}
-VALID_LAYOUT: Final = {"vertical", "horizontal", "grid"}
+VALID_LAYOUT: Final = {"vertical", "horizontal", "grid", "stream"}
 
 VALID_BOX_SIZING: Final = {"border-box", "content-box"}
 VALID_OVERFLOW: Final = {"scroll", "hidden", "auto"}
@@ -76,13 +76,20 @@ VALID_PSEUDO_CLASSES: Final = {
     "nocolor",
     "first-of-type",
     "last-of-type",
+    "first-child",
+    "last-child",
     "odd",
     "even",
+    "empty",
 }
 VALID_OVERLAY: Final = {"none", "screen"}
 VALID_CONSTRAIN: Final = {"inflect", "inside", "none"}
 VALID_KEYLINE: Final = {"none", "thin", "heavy", "double"}
 VALID_HATCH: Final = {"left", "right", "cross", "vertical", "horizontal"}
+VALID_TEXT_WRAP: Final = {"wrap", "nowrap"}
+VALID_TEXT_OVERFLOW: Final = {"clip", "fold", "ellipsis"}
+VALID_EXPAND: Final = {"greedy", "optimal"}
+
 HATCHES: Final = {
     "left": "╲",
     "right": "╱",
