@@ -22,6 +22,7 @@ from rich.style import Style
 from rich.terminal_theme import TerminalTheme
 
 from textual.color import Color
+from textual.constants import DIM_FACTOR
 
 
 class LineFilter(ABC):
@@ -125,7 +126,9 @@ NO_DIM = Style(dim=False)
 
 
 @lru_cache(1024)
-def dim_color(background: RichColor, color: RichColor, factor: float) -> RichColor:
+def dim_color(
+    background: RichColor, color: RichColor, factor: float = DIM_FACTOR
+) -> RichColor:
     """Dim a color by blending towards the background
 
     Args:
@@ -177,13 +180,14 @@ def dim_style(style: Style, background: Color, factor: float) -> Style:
 class DimFilter(LineFilter):
     """Replace dim attributes with modified colors."""
 
-    def __init__(self, dim_factor: float = 0.5) -> None:
+    def __init__(self, dim_factor: float = 0.5, enabled: bool = True) -> None:
         """Initialize the filter.
 
         Args:
             dim_factor: The factor to dim by; 0 is 100% background (i.e. invisible), 1.0 is no change.
         """
         self.dim_factor = dim_factor
+        super().__init__(enabled=enabled)
 
     def apply(self, segments: list[Segment], background: Color) -> list[Segment]:
         """Transform a list of segments.
@@ -226,7 +230,7 @@ class ANSIToTruecolor(LineFilter):
         super().__init__(enabled=enabled)
 
     @lru_cache(1024)
-    def truecolor_style(self, style: Style) -> Style:
+    def truecolor_style(self, style: Style, background: RichColor) -> Style:
         """Replace system colors with truecolor equivalent.
 
         Args:
@@ -237,15 +241,19 @@ class ANSIToTruecolor(LineFilter):
         """
         terminal_theme = self._terminal_theme
         color = style.color
-        if color is not None and color.is_system_defined:
+        if color is not None and color.triplet is None:
             color = RichColor.from_rgb(
                 *color.get_truecolor(terminal_theme, foreground=True)
             )
         bgcolor = style.bgcolor
-        if bgcolor is not None and bgcolor.is_system_defined:
+        if bgcolor is not None and bgcolor.triplet is None:
             bgcolor = RichColor.from_rgb(
                 *bgcolor.get_truecolor(terminal_theme, foreground=False)
             )
+        # Convert dim style to RGB
+        if style.dim and color is not None:
+            color = dim_color(background, color)
+            style += NO_DIM
 
         return style + Style.from_color(color, bgcolor)
 
@@ -262,10 +270,16 @@ class ANSIToTruecolor(LineFilter):
         _Segment = Segment
         truecolor_style = self.truecolor_style
 
+        background_rich_color = background.rich_color
+
         return [
             _Segment(
                 text,
-                None if style is None else truecolor_style(style),
+                (
+                    None
+                    if style is None
+                    else truecolor_style(style, background_rich_color)
+                ),
                 None,
             )
             for text, style, _ in segments
