@@ -39,7 +39,9 @@ from textual._path import (
     _css_path_type_as_list,
     _make_path_object_relative,
 )
+from textual._spatial_map import SpatialMap
 from textual._types import CallbackType
+from textual.actions import SkipAction
 from textual.await_complete import AwaitComplete
 from textual.binding import ActiveBinding, Binding, BindingsMap
 from textual.css.match import match
@@ -47,12 +49,13 @@ from textual.css.parse import parse_selectors
 from textual.css.query import NoMatches, QueryType
 from textual.dom import DOMNode
 from textual.errors import NoWidget
-from textual.geometry import Offset, Region, Size
+from textual.geometry import NULL_OFFSET, Offset, Region, Size
 from textual.keys import key_to_character
 from textual.layout import DockArrangeResult
-from textual.reactive import Reactive
+from textual.reactive import Reactive, var
 from textual.renderables.background_screen import BackgroundScreen
 from textual.renderables.blank import Blank
+from textual.selection import SELECT_ALL, Selection
 from textual.signal import Signal
 from textual.timer import Timer
 from textual.widget import Widget
@@ -145,6 +148,8 @@ class Screen(Generic[ScreenResultType], Widget):
         This CSS applies to the whole app.
     """
 
+    COMPONENT_CLASSES = {"screen--selection"}
+
     DEFAULT_CSS = """
     Screen {
         layout: vertical;
@@ -169,6 +174,9 @@ class Screen(Generic[ScreenResultType], Widget):
                 }
             }
         }
+        .screen--selection {
+            background: $primary 50%;            
+        }
     }
     """
 
@@ -188,6 +196,11 @@ class Screen(Generic[ScreenResultType], Widget):
     you can set the [sub_title][textual.screen.Screen.sub_title] attribute.
     """
 
+    HORIZONTAL_BREAKPOINTS: ClassVar[list[tuple[int, str]]] | None = None
+    """Horizontal breakpoints, will override [App.HORIZONTAL_BREAKPOINTS][textual.app.App.HORIZONTAL_BREAKPOINTS] if not `None`."""
+    VERTICAL_BREAKPOINTS: ClassVar[list[tuple[int, str]]] | None = None
+    """Vertical breakpoints, will override [App.VERTICAL_BREAKPOINTS][textual.app.App.VERTICAL_BREAKPOINTS] if not `None`."""
+
     focused: Reactive[Widget | None] = Reactive(None)
     """The focused [widget][textual.widget.Widget] or `None` for no focus.
     To set focus, do not update this value directly. Use [set_focus][textual.screen.Screen.set_focus] instead."""
@@ -213,9 +226,27 @@ class Screen(Generic[ScreenResultType], Widget):
     maximized: Reactive[Widget | None] = Reactive(None, layout=True)
     """The currently maximized widget, or `None` for no maximized widget."""
 
+    selections: var[dict[Widget, Selection]] = var(dict)
+    """Map of widgets and selected ranges."""
+
+    _selecting = var(False)
+    """Indicates mouse selection is in progress."""
+
+    _box_select = var(False)
+    """Should text selection be limited to a box?"""
+
+    _select_start: Reactive[tuple[Widget, Offset, Offset] | None] = Reactive(None)
+    """Tuple of (widget, screen offset, text offset) where selection started."""
+    _select_end: Reactive[tuple[Widget, Offset, Offset] | None] = Reactive(None)
+    """Tuple of (widget, screen offset, text offset) where selection ends."""
+
+    _mouse_down_offset: var[Offset | None] = var(None)
+    """Last mouse down screen offset, or `None` if the mouse is up."""
+
     BINDINGS = [
         Binding("tab", "app.focus_next", "Focus Next", show=False),
         Binding("shift+tab", "app.focus_previous", "Focus Previous", show=False),
+        Binding("ctrl+c", "screen.copy_text", "Copy selected text", show=False),
     ]
 
     def __init__(
@@ -261,11 +292,14 @@ class Screen(Generic[ScreenResultType], Widget):
         )
         """The signal that is published when the screen's layout is refreshed."""
 
-        self._bindings_updated = False
-        """Indicates that a binding update was requested."""
         self.bindings_updated_signal: Signal[Screen] = Signal(self, "bindings_updated")
         """A signal published when the bindings have been updated"""
 
+        self.text_selection_started_signal: Signal[Screen] = Signal(
+            self, "selection_started"
+        )
+        """A signal published when text selection has started."""
+
         self._css_update_count = -1
         """Track updates to CSS."""
 
@@ -313,10 +347,17 @@ class Screen(Generic[ScreenResultType], Widget):
     def _watch_stack_updates(self):
         self.refresh_bindings()
 
+    async def _watch_selections(
+        self,
+        old_selections: dict[Widget, Selection],
+        selections: dict[Widget, Selection],
+    ):
+        for widget in old_selections.keys() | selections.keys():
+            widget.selection_updated(selections.get(widget, None))
+
     def refresh_bindings(self) -> None:
         """Call to request a refresh of bindings."""
-        self._bindings_updated = True
-        self.check_idle()
+        self.bindings_updated_signal.publish(self)
 
     def _watch_maximized(
         self, previously_maximized: Widget | None, maximized: Widget | None
@@ -472,6 +513,7 @@ class Screen(Generic[ScreenResultType], Widget):
             ),
             size,
             self.screen.size,
+            False,
         )
 
         return arrangement
@@ -484,6 +526,11 @@ class Screen(Generic[ScreenResultType], Widget):
         except Exception:
             return False
 
+    @property
+    def allow_select(self) -> bool:
+        """Check if this widget permits text selection."""
+        return self.ALLOW_SELECT
+
     def render(self) -> RenderableType:
         """Render method inherited from widget, used to render the screen's background.
 
@@ -583,6 +630,20 @@ class Screen(Generic[ScreenResultType], Widget):
         """
         return self._compositor.get_style_at(x, y)
 
+    def get_widget_and_offset_at(
+        self, x: int, y: int
+    ) -> tuple[Widget | None, Offset | None]:
+        """Get the widget under a given coordinate, and an offset within the original content.
+
+        Args:
+            x: X Coordinate.
+            y: Y Coordinate.
+
+        Returns:
+            Tuple of Widget and Offset, both of which may be None.
+        """
+        return self._compositor.get_widget_and_offset_at(x, y)
+
     def find_widget(self, widget: Widget) -> MapGeometry:
         """Get the screen region of a Widget.
 
@@ -597,6 +658,24 @@ class Screen(Generic[ScreenResultType], Widget):
         """
         return self._compositor.find_widget(widget)
 
+    def clear_selection(self) -> None:
+        """Clear any selected text."""
+        self.selections = {}
+        self._select_start = None
+        self._select_end = None
+
+    def _select_all_in_widget(self, widget: Widget) -> None:
+        """Select a widget and all it's children.
+
+        Args:
+            widget: Widget to select.
+        """
+        select_all = SELECT_ALL
+        self.selections = {
+            widget: select_all,
+            **{child: select_all for child in widget.query("*")},
+        }
+
     @property
     def focus_chain(self) -> list[Widget]:
         """A list of widgets that may receive focus, in focus order."""
@@ -665,8 +744,6 @@ class Screen(Generic[ScreenResultType], Widget):
                 the CSS selectors given in the argument.
         """
 
-        # TODO: This shouldn't be required
-        self._compositor._full_map_invalidated = True
         if not isinstance(selector, str):
             selector = selector.__name__
         selector_set = parse_selectors(selector)
@@ -782,6 +859,32 @@ class Screen(Generic[ScreenResultType], Widget):
                 self.scroll_to_widget, self.focused, animate=False, center=True
             )
 
+    def get_selected_text(self) -> str | None:
+        """Get text under selection.
+
+        Returns:
+            Selected text, or `None` if no text was selected.
+        """
+        if not self.selections:
+            return None
+
+        widget_text: list[str] = []
+        for widget, selection in self.selections.items():
+            selected_text_in_widget = widget.get_selection(selection)
+            if selected_text_in_widget is not None:
+                widget_text.extend(selected_text_in_widget)
+
+        selected_text = "".join(widget_text)
+        return selected_text
+
+    def action_copy_text(self) -> None:
+        """Copy selected text to clipboard."""
+        selection = self.get_selected_text()
+        if selection is None:
+            # No text selected
+            raise SkipAction()
+        self.app.copy_to_clipboard(selection)
+
     def action_maximize(self) -> None:
         """Action to maximize the currently focused widget."""
         if self.focused is not None:
@@ -795,6 +898,20 @@ class Screen(Generic[ScreenResultType], Widget):
         """Action to remove focus (if set)."""
         self.set_focus(None)
 
+    async def action_focus(self, selector: str) -> None:
+        """An [action](/guide/actions) to focus the given widget.
+
+        Args:
+            selector: Selector of widget to focus (first match).
+        """
+        try:
+            node = self.query(selector).first()
+        except NoMatches:
+            pass
+        else:
+            if isinstance(node, Widget):
+                self.set_focus(node)
+
     def _reset_focus(
         self, widget: Widget, avoiding: list[Widget] | None = None
     ) -> None:
@@ -870,16 +987,22 @@ class Screen(Generic[ScreenResultType], Widget):
                     widgets.update(widget.walk_children(with_self=True))
                     break
         if widgets:
-            self.app.stylesheet.update_nodes(
-                [widget for widget in widgets if widget._has_focus_within], animate=True
-            )
+            self.app.stylesheet.update_nodes(widgets, animate=True)
 
-    def set_focus(self, widget: Widget | None, scroll_visible: bool = True) -> None:
+    def set_focus(
+        self,
+        widget: Widget | None,
+        scroll_visible: bool = True,
+        from_app_focus: bool = False,
+    ) -> None:
         """Focus (or un-focus) a widget. A focused widget will receive key events first.
 
         Args:
             widget: Widget to focus, or None to un-focus.
-            scroll_visible: Scroll widget in to view.
+            scroll_visible: Scroll widget into view.
+            from_app_focus: True if this focus is due to the app itself having regained
+                focus. False if the focus is being set because a widget within the app
+                regained focus.
         """
         if widget is self.focused:
             # Widget is already focused
@@ -904,7 +1027,7 @@ class Screen(Generic[ScreenResultType], Widget):
                 # Change focus
                 self.focused = widget
                 # Send focus event
-                widget.post_message(events.Focus())
+                widget.post_message(events.Focus(from_app_focus=from_app_focus))
                 focused = widget
 
                 if scroll_visible:
@@ -919,7 +1042,7 @@ class Screen(Generic[ScreenResultType], Widget):
                 self.log.debug(widget, "was focused")
 
         self._update_focus_styles(focused, blurred)
-        self.refresh_bindings()
+        self.call_after_refresh(self.refresh_bindings)
 
     def _extend_compose(self, widgets: list[Widget]) -> None:
         """Insert Textual's own internal widgets.
@@ -940,32 +1063,22 @@ class Screen(Generic[ScreenResultType], Widget):
         self.screen_layout_refresh_signal.subscribe(
             self, self._maybe_clear_tooltip, immediate=True
         )
-        self.refresh_bindings()
-        # Send this signal so we don't get an initial frame with no bindings in the footer
-        self.bindings_updated_signal.publish(self)
 
     async def _on_idle(self, event: events.Idle) -> None:
         # Check for any widgets marked as 'dirty' (needs a repaint)
         event.prevent_default()
+        if not self.app._batch_count and self.is_current:
+            if (
+                self._layout_required
+                or self._scroll_required
+                or self._repaint_required
+                or self._recompose_required
+                or self._dirty_widgets
+            ):
+                self._update_timer.resume()
+                return
 
-        try:
-            if not self.app._batch_count and self.is_current:
-                if (
-                    self._layout_required
-                    or self._scroll_required
-                    or self._repaint_required
-                    or self._recompose_required
-                    or self._dirty_widgets
-                ):
-                    self._update_timer.resume()
-                    return
-
-            await self._invoke_and_clear_callbacks()
-        finally:
-            if self._bindings_updated:
-                self._bindings_updated = False
-                if self.is_attached and not self.app._exit:
-                    self.app.call_later(self.bindings_updated_signal.publish, self)
+        await self._invoke_and_clear_callbacks()
 
     def _compositor_refresh(self) -> None:
         """Perform a compositor refresh."""
@@ -1218,22 +1331,21 @@ class Screen(Generic[ScreenResultType], Widget):
         inline_height = min(self.app.size.height, inline_height)
         return inline_height
 
-    def _screen_resized(self, size: Size):
+    def _screen_resized(self, size: Size) -> None:
         """Called by App when the screen is resized."""
-        if self.stack_updates:
-            self._compositor_refresh()
+        if self.stack_updates and self.is_attached:
             self._refresh_layout(size)
 
     def _on_screen_resume(self) -> None:
         """Screen has resumed."""
         if self.app.SUSPENDED_SCREEN_CLASS:
             self.remove_class(self.app.SUSPENDED_SCREEN_CLASS)
+
         self.stack_updates += 1
+
         self.app._refresh_notifications()
         size = self.app.size
 
-        self._refresh_layout(size)
-        self.refresh()
         # Only auto-focus when the app has focus (textual-web only)
         if self.app.app_focus:
             auto_focus = (
@@ -1245,6 +1357,12 @@ class Screen(Generic[ScreenResultType], Widget):
                         self.set_focus(widget)
                         break
 
+        if self.is_attached:
+            self._compositor_refresh()
+            self.app.stylesheet.update(self)
+            self._refresh_layout(size)
+            self.refresh()
+
     def _on_screen_suspend(self) -> None:
         """Screen has suspended."""
         if self.app.SUSPENDED_SCREEN_CLASS:
@@ -1259,6 +1377,41 @@ class Screen(Generic[ScreenResultType], Widget):
         for screen in self.app._background_screens:
             screen._screen_resized(event.size)
 
+        horizontal_breakpoints = (
+            self.app.HORIZONTAL_BREAKPOINTS
+            if self.HORIZONTAL_BREAKPOINTS is None
+            else self.HORIZONTAL_BREAKPOINTS
+        ) or []
+
+        vertical_breakpoints = (
+            self.app.VERTICAL_BREAKPOINTS
+            if self.VERTICAL_BREAKPOINTS is None
+            else self.VERTICAL_BREAKPOINTS
+        ) or []
+
+        width, height = event.size
+        if horizontal_breakpoints:
+            self._set_breakpoints(width, horizontal_breakpoints)
+        if vertical_breakpoints:
+            self._set_breakpoints(height, vertical_breakpoints)
+
+    def _set_breakpoints(
+        self, dimension: int, breakpoints: list[tuple[int, str]]
+    ) -> None:
+        """Set horizontal or vertical breakpoints.
+
+        Args:
+            dimension: Either the width or the height.
+            breakpoints: A list of breakpoints.
+
+        """
+        class_names = [class_name for _breakpoint, class_name in breakpoints]
+        self.remove_class(*class_names)
+        for breakpoint, class_name in sorted(breakpoints, reverse=True):
+            if dimension >= breakpoint:
+                self.add_class(class_name)
+                return
+
     def _update_tooltip(self, widget: Widget) -> None:
         """Update the content of the tooltip."""
         try:
@@ -1381,16 +1534,16 @@ class Screen(Generic[ScreenResultType], Widget):
         """
         return events.MouseMove(
             widget,
-            event.x - region.x,
-            event.y - region.y,
-            event.delta_x,
-            event.delta_y,
+            event._x - region.x,
+            event._y - region.y,
+            event._delta_x,
+            event._delta_y,
             event.button,
             event.shift,
             event.meta,
             event.ctrl,
-            screen_x=event.screen_x,
-            screen_y=event.screen_y,
+            screen_x=event._screen_x,
+            screen_y=event._screen_y,
             style=event.style,
         )
 
@@ -1406,7 +1559,60 @@ class Screen(Generic[ScreenResultType], Widget):
             event.style = self.get_style_at(event.screen_x, event.screen_y)
             self._handle_mouse_move(event)
 
+            if self._selecting:
+                self._box_select = event.shift
+                select_widget, select_offset = self.get_widget_and_offset_at(
+                    event.x, event.y
+                )
+                if (
+                    self._select_end is not None
+                    and select_offset is None
+                    and event.y > self._select_end[1].y
+                ):
+                    end_widget = self._select_end[0]
+                    select_offset = end_widget.content_region.bottom_right_inclusive
+                    self._select_end = (end_widget, event.offset, select_offset)
+
+                elif (
+                    select_widget is not None
+                    and select_widget.allow_select
+                    and select_offset is not None
+                ):
+                    self._select_end = (select_widget, event.offset, select_offset)
+
         elif isinstance(event, events.MouseEvent):
+            if isinstance(event, events.MouseUp):
+                if (
+                    self._mouse_down_offset is not None
+                    and self._mouse_down_offset == event.screen_offset
+                ):
+                    self.clear_selection()
+                self._mouse_down_offset = None
+                self._selecting = False
+
+            elif isinstance(event, events.MouseDown) and not self.app.mouse_captured:
+                self._box_select = event.shift
+                self._mouse_down_offset = event.screen_offset
+                select_widget, select_offset = self.get_widget_and_offset_at(
+                    event.screen_x, event.screen_y
+                )
+                if (
+                    select_widget is not None
+                    and select_widget.allow_select
+                    and self.screen.allow_select
+                    and self.app.ALLOW_SELECT
+                ):
+                    self._selecting = True
+                    if select_widget is not None and select_offset is not None:
+                        self.text_selection_started_signal.publish(self)
+                        self._select_start = (
+                            select_widget,
+                            event.screen_offset,
+                            select_offset,
+                        )
+                else:
+                    self._selecting = False
+
             try:
                 if self.app.mouse_captured:
                     widget = self.app.mouse_captured
@@ -1432,6 +1638,112 @@ class Screen(Generic[ScreenResultType], Widget):
         else:
             self.post_message(event)
 
+    def _key_escape(self) -> None:
+        self.clear_selection()
+
+    def _watch__select_end(
+        self, select_end: tuple[Widget, Offset, Offset] | None
+    ) -> None:
+        """When select_end changes, we need to compute which widgets and regions are selected.
+
+        Args:
+            select_end: The end selection.
+        """
+
+        if select_end is None or self._select_start is None:
+            # Nothing to select
+            return
+
+        select_start = self._select_start
+        start_widget, screen_start, start_offset = select_start
+        end_widget, screen_end, end_offset = select_end
+        if start_widget is end_widget:
+            # Simplest case, selection starts and ends on the same widget
+            self.selections = {
+                start_widget: Selection.from_offsets(start_offset, end_offset)
+            }
+            return
+
+        select_start, select_end = sorted(
+            [select_start, select_end],
+            key=lambda selection: (selection[0].region.offset.transpose),
+        )
+
+        start_widget, _screen_start, start_offset = select_start
+        end_widget, _screen_end, end_offset = select_end
+
+        select_regions: list[Region] = []
+        start_region = start_widget.content_region
+        end_region = end_widget.content_region
+        if end_region.y <= start_region.bottom or self._box_select:
+            select_regions.append(Region.union(start_region, end_region))
+        else:
+            try:
+                container_region = Region.from_union(
+                    [
+                        start_widget.select_container.content_region,
+                        end_widget.select_container.content_region,
+                    ]
+                )
+            except NoMatches:
+                return
+
+            start_region = Region.from_corners(
+                start_region.x,
+                start_region.y,
+                container_region.right,
+                start_region.bottom,
+            )
+            end_region = Region.from_corners(
+                container_region.x,
+                end_region.y,
+                end_region.right,
+                end_region.bottom,
+            )
+            select_regions.append(start_region)
+            select_regions.append(end_region)
+            mid_height = end_region.y - start_region.bottom
+            if mid_height > 0:
+                mid_region = Region.from_corners(
+                    container_region.x,
+                    start_region.bottom,
+                    container_region.right,
+                    start_region.bottom + mid_height,
+                )
+                select_regions.append(mid_region)
+
+        spatial_map: SpatialMap[Widget] = SpatialMap()
+        spatial_map.insert(
+            [
+                (widget.region, NULL_OFFSET, False, False, widget)
+                for widget in self._compositor.visible_widgets.keys()
+            ]
+        )
+
+        highlighted_widgets: set[Widget] = set()
+        for region in select_regions:
+            covered_widgets = spatial_map.get_values_in_region(region)
+            covered_widgets = [
+                widget
+                for widget in covered_widgets
+                if region.overlaps(widget.content_region)
+            ]
+            highlighted_widgets.update(covered_widgets)
+        highlighted_widgets -= {self, start_widget, end_widget}
+
+        select_all = SELECT_ALL
+        self.selections = {
+            start_widget: Selection(start_offset, None),
+            **{
+                widget: select_all
+                for widget in sorted(
+                    highlighted_widgets,
+                    key=lambda widget: widget.content_region.offset.transpose,
+                )
+            },
+            end_widget: Selection(None, end_offset),
+        }
+
     def dismiss(self, result: ScreenResultType | None = None) -> AwaitComplete:
         """Dismiss the screen, optionally with a result.
 
