@@ -13,7 +13,7 @@ from markdown_it.token import Token
 from rich.text import Text
 from typing_extensions import TypeAlias
 
-from textual._slug import TrackedSlugs, slug
+from textual._slug import TrackedSlugs, slug_for_tcss_id
 from textual.app import ComposeResult
 from textual.await_complete import AwaitComplete
 from textual.containers import Horizontal, Vertical, VerticalScroll
@@ -354,6 +354,8 @@ class MarkdownBlock(Static):
 class MarkdownHeader(MarkdownBlock):
     """Base class for a Markdown header."""
 
+    LEVEL = 0
+
     DEFAULT_CSS = """
     MarkdownHeader {
         color: $text;
@@ -366,6 +368,8 @@ class MarkdownHeader(MarkdownBlock):
 class MarkdownH1(MarkdownHeader):
     """An H1 Markdown header."""
 
+    LEVEL = 1
+
     DEFAULT_CSS = """
     MarkdownH1 {
         content-align: center middle;
@@ -379,6 +383,8 @@ class MarkdownH1(MarkdownHeader):
 class MarkdownH2(MarkdownHeader):
     """An H2 Markdown header."""
 
+    LEVEL = 2
+
     DEFAULT_CSS = """
     MarkdownH2 {
         color: $markdown-h2-color;
@@ -391,6 +397,8 @@ class MarkdownH2(MarkdownHeader):
 class MarkdownH3(MarkdownHeader):
     """An H3 Markdown header."""
 
+    LEVEL = 3
+
     DEFAULT_CSS = """
     MarkdownH3 {
         color: $markdown-h3-color;
@@ -405,6 +413,8 @@ class MarkdownH3(MarkdownHeader):
 class MarkdownH4(MarkdownHeader):
     """An H4 Markdown header."""
 
+    LEVEL = 4
+
     DEFAULT_CSS = """
     MarkdownH4 {
         color: $markdown-h4-color;
@@ -418,6 +428,8 @@ class MarkdownH4(MarkdownHeader):
 class MarkdownH5(MarkdownHeader):
     """An H5 Markdown header."""
 
+    LEVEL = 5
+
     DEFAULT_CSS = """
     MarkdownH5 {
         color: $markdown-h5-color;
@@ -431,6 +443,8 @@ class MarkdownH5(MarkdownHeader):
 class MarkdownH6(MarkdownHeader):
     """An H6 Markdown header."""
 
+    LEVEL = 6
+
     DEFAULT_CSS = """
     MarkdownH6 {
         color: $markdown-h6-color;
@@ -551,7 +565,7 @@ class MarkdownOrderedList(MarkdownList):
 
     MarkdownOrderedList Vertical {
         height: auto;
-        width: 1fr;        
+        width: 1fr;
     }
     """
 
@@ -574,7 +588,7 @@ class MarkdownOrderedList(MarkdownList):
                 bullet.symbol = f"{number}{suffix}".rjust(symbol_size + 1)
                 yield Horizontal(bullet, Vertical(*block._blocks))
 
-        # self._blocks.clear()
+        self._blocks.clear()
 
 
 class MarkdownTableCellContents(Static):
@@ -593,29 +607,28 @@ class MarkdownTableContent(Widget):
 
     DEFAULT_CSS = """
     MarkdownTableContent {
-        width: 1fr;        
+        width: 1fr;
         height: auto;
-        layout: grid;        
-        grid-columns: auto;   
-        grid-rows: auto;    
-        grid-gutter: 1 1; 
-                
+        layout: grid;
+        grid-columns: auto;
+        grid-rows: auto;
+        grid-gutter: 1 1;
+
         & > .cell {
             margin: 0 0;
             height: auto;
             padding: 0 1;
             text-overflow: ellipsis;
-            
         }
         & > .header {
             height: auto;
             margin: 0 0;
             padding: 0 1;
             color: $primary;
-            text-overflow: ellipsis;            
-            content-align: left bottom;            
+            text-overflow: ellipsis;
+            content-align: left bottom;
         }
-        keyline: thin $foreground 20%;                
+        keyline: thin $foreground 20%;
     }
     MarkdownTableContent > .markdown-table--header {
         text-style: bold;
@@ -689,11 +702,11 @@ class MarkdownTable(MarkdownBlock):
 
     DEFAULT_CSS = """
     MarkdownTable {
-        width: 1fr;      
-        margin-bottom: 1;      
+        width: 1fr;
+        margin-bottom: 1;
         &:light {
             background: white 30%;
-        }        
+        }
     }
     """
 
@@ -782,7 +795,7 @@ class MarkdownBullet(Widget):
     DEFAULT_CSS = """
     MarkdownBullet {
         width: auto;
-        color: $text-primary;        
+        color: $text-primary;
         &:light {
             color: $text-secondary;
         }
@@ -839,15 +852,15 @@ class MarkdownFence(MarkdownBlock):
         scrollbar-size-horizontal: 0;
         scrollbar-size-vertical: 0;
         width: 1fr;
-        height: auto;      
+        height: auto;
         color: rgb(210,210,210);
         background: black 10%;
         &:light {
             background: white 30%;
-        }        
+        }
         & > Label {
             padding: 1 2;
-        }        
+        }
     }
     """
 
@@ -890,15 +903,15 @@ NUMERALS = " ⅠⅡⅢⅣⅤⅥ"
 
 class Markdown(Widget):
     DEFAULT_CSS = """
-    Markdown {        
-        height: auto;        
+    Markdown {
+        height: auto;
         padding: 0 2 0 2;
         layout: vertical;
-        color: $foreground;       
+        color: $foreground;
         overflow-y: hidden;
-              
+
         MarkdownBlock {
-            &:dark > .code_inline {                
+            &:dark > .code_inline {
                 background: $warning 10%;
                 color: $text-warning 95%;
             }
@@ -907,7 +920,7 @@ class Markdown(Widget):
                 color: $text-error 95%;
             }
             & > .em {
-                text-style: italic;                        
+                text-style: italic;
             }
             & > .strong {
                 text-style: bold;
@@ -916,10 +929,7 @@ class Markdown(Widget):
                 text-style: strike;
             }
         }
-    
     }
-    
-    
     """
 
     BULLETS = ["• ", "▪ ", "‣ ", "⭑ ", "◦ "]
@@ -974,11 +984,22 @@ class Markdown(Widget):
         self._initial_markdown: str | None = markdown
         self._markdown = ""
         self._parser_factory = parser_factory
-        self._table_of_contents: TableOfContentsType = []
+        self._table_of_contents: TableOfContentsType | None = None
         self._open_links = open_links
         self._last_parsed_line = 0
         self._theme = ""
 
+    @property
+    def table_of_contents(self) -> TableOfContentsType:
+        """The document's table of contents."""
+        if self._table_of_contents is None:
+            self._table_of_contents = [
+                (header.LEVEL, header._content.plain, header.id)
+                for header in self.children
+                if isinstance(header, MarkdownHeader)
+            ]
+        return self._table_of_contents
+
     class TableOfContentsUpdated(Message):
         """The table of contents was updated."""
 
@@ -1068,13 +1089,13 @@ class Markdown(Widget):
 
     @classmethod
     def get_stream(cls, markdown: Markdown) -> MarkdownStream:
-        """Get a [MarkdownStream][textual.widgets._markdown.MarkdownStream] instance stream Markdown in the background.
+        """Get a [MarkdownStream][textual.widgets.markdown.MarkdownStream] instance to stream Markdown in the background.
 
         If you append to the Markdown document many times a second, it is possible the widget won't
         be able to update as fast as you write (occurs around 20 appends per second). It will still
         work, but the user will have to wait for the UI to catch up after the document has be retrieved.
 
-        Using a [MarkdownStream][textual.widgets._markdown.MarkdownStream] will combine several updates in to one
+        Using a [MarkdownStream][textual.widgets.markdown.MarkdownStream] will combine several updates in to one
         as necessary to keep up with the incoming data.
 
         example:
@@ -1182,16 +1203,11 @@ class Markdown(Widget):
         """
         return None
 
-    def _parse_markdown(
-        self,
-        tokens: Iterable[Token],
-        table_of_contents: TableOfContentsType,
-    ) -> Iterable[MarkdownBlock]:
+    def _parse_markdown(self, tokens: Iterable[Token]) -> Iterable[MarkdownBlock]:
         """Create a stream of MarkdownBlock widgets from markdown.
 
         Args:
             tokens: List of tokens.
-            table_of_contents: List to store table of contents.
 
         Yields:
             Widgets for mounting.
@@ -1251,10 +1267,9 @@ class Markdown(Widget):
             elif token_type.endswith("_close"):
                 block = stack.pop()
                 if token.type == "heading_close":
-                    heading = block._content.plain
-                    level = int(token.tag[1:])
-                    block.id = f"{slug(heading)}-{len(table_of_contents) + 1}"
-                    table_of_contents.append((level, heading, block.id))
+                    block.id = (
+                        f"heading-{slug_for_tcss_id(block._content.plain)}-{id(block)}"
+                    )
                 if stack:
                     stack[-1]._blocks.append(block)
                 else:
@@ -1262,7 +1277,9 @@ class Markdown(Widget):
             elif token_type == "inline":
                 stack[-1].build_from_token(token)
             elif token_type in ("fence", "code_block"):
-                fence = get_block_class(token_type)(self, token, token.content.rstrip())
+                fence_class = get_block_class(token_type)
+                assert issubclass(fence_class, MarkdownFence)
+                fence = fence_class(self, token, token.content.rstrip())
                 if stack:
                     stack[-1]._blocks.append(fence)
                 else:
@@ -1276,13 +1293,21 @@ class Markdown(Widget):
                         yield external
 
     def _build_from_source(self, markdown: str) -> list[MarkdownBlock]:
+        """Build blocks from markdown source.
+
+        Args:
+            markdown: A Markdown document, or partial document.
+
+        Returns:
+            A list of MarkdownBlock instances.
+        """
         parser = (
             MarkdownIt("gfm-like")
             if self._parser_factory is None
             else self._parser_factory()
         )
         tokens = parser.parse(markdown)
-        return list(self._parse_markdown(tokens, []))
+        return list(self._parse_markdown(tokens))
 
     def update(self, markdown: str) -> AwaitComplete:
         """Update the document with new Markdown.
@@ -1300,9 +1325,9 @@ class Markdown(Widget):
             else self._parser_factory()
         )
 
-        table_of_contents: TableOfContentsType = []
         markdown_block = self.query("MarkdownBlock")
         self._markdown = markdown
+        self._table_of_contents = None
 
         async def await_update() -> None:
             """Update in batches."""
@@ -1333,7 +1358,7 @@ class Markdown(Widget):
                             await self.mount_all(batch)
                         removed = True
 
-                for block in self._parse_markdown(tokens, table_of_contents):
+                for block in self._parse_markdown(tokens):
                     batch.append(block)
                     if len(batch) == BATCH_SIZE:
                         await mount_batch(batch)
@@ -1343,13 +1368,13 @@ class Markdown(Widget):
                 if not removed:
                     await markdown_block.remove()
 
-            if table_of_contents != self._table_of_contents:
-                self._table_of_contents = table_of_contents
-                self.post_message(
-                    Markdown.TableOfContentsUpdated(
-                        self, self._table_of_contents
-                    ).set_sender(self)
-                )
+            lines = markdown.splitlines()
+            self._last_parsed_line = len(lines) - (1 if lines and lines[-1] else 0)
+            self.post_message(
+                Markdown.TableOfContentsUpdated(
+                    self, self.table_of_contents
+                ).set_sender(self)
+            )
 
         return AwaitComplete(await_update())
 
@@ -1368,7 +1393,6 @@ class Markdown(Widget):
             else self._parser_factory()
         )
 
-        table_of_contents: TableOfContentsType = []
         self._markdown = self.source + markdown
         updated_source = "".join(
             self._markdown.splitlines(keepends=True)[self._last_parsed_line :]
@@ -1387,7 +1411,10 @@ class Markdown(Widget):
                         self._last_parsed_line += token.map[0]
                         break
 
-                new_blocks = list(self._parse_markdown(tokens, table_of_contents))
+                new_blocks = list(self._parse_markdown(tokens))
+                any_headers = any(
+                    isinstance(block, MarkdownHeader) for block in new_blocks
+                )
                 for block in new_blocks:
                     start, end = block.source_range
                     block.source_range = (
@@ -1409,12 +1436,13 @@ class Markdown(Widget):
                     if new_blocks:
                         await self.mount_all(new_blocks)
 
-                self._table_of_contents = table_of_contents
-                self.post_message(
-                    Markdown.TableOfContentsUpdated(
-                        self, self._table_of_contents
-                    ).set_sender(self)
-                )
+                if any_headers:
+                    self._table_of_contents = None
+                    self.post_message(
+                        Markdown.TableOfContentsUpdated(
+                            self, self.table_of_contents
+                        ).set_sender(self)
+                    )
 
         return AwaitComplete(await_append())
 
